.SystemUI-LayoutSidebar {
    width: 18vw;
    flex-shrink: 0;
    margin-right: 1vw;
}

.SystemUI-LayoutSidebar-paper {
    width: 19vw;
    background: linear-gradient(to right bottom, #222, #444) !important;
    color: #eee !important;
    padding: .35vmax 0 !important;
    box-shadow: 0 0 1vmax #0004 !important;
    margin: .5vmax;
    border-radius: 7px;
    border: 1px solid #111 !important;
    height: calc(100% - 2vmax) !important;
}

.SystemUI-LayoutSidebar-container {
    overflow: auto;
}

.SystemUI-LayoutSidebar-folder {
    font-size: 0.8em;
    background-color: #0002;
    border: 1px solid #0009;
    border-radius: .5vmax;
    margin: 4px;
}

.SystemUI-LayoutSidebar a {
    text-decoration: none;
    color: inherit;
}

.SystemUI-LayoutSidebar a:hover {
    background-color: #0005 !important;
}

.SystemUI-LayoutSidebar-item {
    text-shadow: 0 1px 1px #0009;
    padding: .25vmax 0;
}

.SystemUI-LayoutSidebar-item:hover {
    background-color: #0005 !important;
}

.SystemUI-LayoutSidebar-icon {
    width: 1.75vw;
    height: 1.75vw;
}

.SystemUI-LayoutSidebar-folder .SystemUI-LayoutSidebar-icon {
    width: 1.25vw;
    height: 1.25vw;
}
